@@ -50,6 +50,7 @@ struct GstRTSPOnvifMediaFactoryPrivate
   GMutex lock;
   gchar *backchannel_launch;
   guint backchannel_bandwidth;
+  gboolean has_replay_support;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GstRTSPOnvifMediaFactory,
@@ -359,7 +360,7 @@ gst_rtsp_onvif_media_factory_init (GstRTSPOnvifMediaFactory * factory)
 /**
  * gst_rtsp_onvif_media_factory_set_backchannel_launch:
  * @factory: a #GstRTSPMediaFactory
- * @launch: the launch description
+ * @launch: (nullable): the launch description
  *
  * The gst_parse_launch() line to use for constructing the ONVIF backchannel
  * pipeline in the default prepare vmethod if requested by the client.
@@ -399,7 +400,7 @@ gst_rtsp_onvif_media_factory_set_backchannel_launch (GstRTSPOnvifMediaFactory *
  * Get the gst_parse_launch() pipeline description that will be used in the
  * default prepare vmethod for generating the ONVIF backchannel pipeline.
  *
- * Returns: (transfer full): the configured backchannel launch description. g_free() after
+ * Returns: (transfer full) (nullable): the configured backchannel launch description. g_free() after
  * usage.
  *
  * Since: 1.14
@@ -445,6 +446,42 @@ gst_rtsp_onvif_media_factory_has_backchannel_support (GstRTSPOnvifMediaFactory *
   return FALSE;
 }
 
+/**
+ * gst_rtsp_onvif_media_factory_has_replay_support:
+ *
+ * Returns: %TRUE if ONVIF replay is supported by the media factory.
+ *
+ * Since: 1.18
+ */
+gboolean
+gst_rtsp_onvif_media_factory_has_replay_support (GstRTSPOnvifMediaFactory *
+    factory)
+{
+  gboolean has_replay_support;
+
+  g_mutex_lock (&factory->priv->lock);
+  has_replay_support = factory->priv->has_replay_support;
+  g_mutex_unlock (&factory->priv->lock);
+
+  return has_replay_support;
+}
+
+/**
+ * gst_rtsp_onvif_media_factory_set_replay_support:
+ *
+ * Set to %TRUE if ONVIF replay is supported by the media factory.
+ *
+ * Since: 1.18
+ */
+void
+gst_rtsp_onvif_media_factory_set_replay_support (GstRTSPOnvifMediaFactory *
+    factory, gboolean has_replay_support)
+{
+  g_mutex_lock (&factory->priv->lock);
+  factory->priv->has_replay_support = has_replay_support;
+  g_mutex_unlock (&factory->priv->lock);
+}
+
 /**
  * gst_rtsp_onvif_media_factory_set_backchannel_bandwidth:
  * @factory: a #GstRTSPMediaFactory
