@@ -280,8 +280,10 @@ gst_rtsp_sdp_make_media (GstSDPMessage * sdp, GstSDPInfo * info,
   mikey_msg = gst_mikey_message_new_from_caps (caps);
   if (mikey_msg) {
     /* add policy '0' for all sending SSRC */
-    if (!mikey_add_crypto_sessions (stream, mikey_msg))
+    if (!mikey_add_crypto_sessions (stream, mikey_msg)) {
+      gst_mikey_message_unref (mikey_msg);
       goto crypto_sessions_error;
+    }
 
     base64 = gst_mikey_message_base64_encode (mikey_msg);
     if (base64) {
@@ -295,7 +297,7 @@ gst_rtsp_sdp_make_media (GstSDPMessage * sdp, GstSDPInfo * info,
   }
 
   /* RFC 7273 clock signalling */
-  {
+  if (gst_rtsp_stream_is_sender (stream)) {
     GstBin *joined_bin = gst_rtsp_stream_get_joined_bin (stream);
     GstClock *clock = gst_element_get_clock (GST_ELEMENT_CAST (joined_bin));
     gchar *ts_refclk = NULL;
@@ -305,9 +307,9 @@ gst_rtsp_sdp_make_media (GstSDPMessage * sdp, GstSDPInfo * info,
     GstRTSPPublishClockMode publish_clock_mode =
         gst_rtsp_stream_get_publish_clock_mode (stream);
 
-    if (gst_rtsp_stream_is_sender (stream))
-      gst_rtsp_stream_get_rtpinfo (stream, &rtptime, NULL, &clock_rate,
-          &running_time);
+    if (!gst_rtsp_stream_get_rtpinfo (stream, &rtptime, NULL, &clock_rate,
+            &running_time))
+      goto clock_signalling_cleanup;
     base_time = gst_element_get_base_time (GST_ELEMENT_CAST (joined_bin));
     g_assert (base_time != GST_CLOCK_TIME_NONE);
     clock_time = running_time + base_time;
@@ -374,6 +376,7 @@ gst_rtsp_sdp_make_media (GstSDPMessage * sdp, GstSDPInfo * info,
         }
       }
     }
+  clock_signalling_cleanup:
     if (clock)
       gst_object_unref (clock);
 
@@ -462,6 +465,35 @@ gst_rtsp_sdp_make_media (GstSDPMessage * sdp, GstSDPInfo * info,
     }
   }
 
+  /* RFC5576 Source-specific media attributes */
+  {
+    GObject *session;
+    guint ssrc;
+    GstStructure *sdes;
+    const gchar *cname;
+    gchar *ssrc_cname;
+
+    session = gst_rtsp_stream_get_rtpsession (stream);
+    if (session) {
+      g_object_get (session, "sdes", &sdes, NULL);
+
+      cname = gst_structure_get_string (sdes, "cname");
+      gst_rtsp_stream_get_ssrc (stream, &ssrc);
+
+      if (cname) {
+        ssrc_cname = g_strdup_printf ("%u cname:%s", ssrc, cname);
+        gst_sdp_media_add_attribute (smedia, "ssrc", ssrc_cname);
+        g_free (ssrc_cname);
+      } else {
+        GST_ERROR ("unable to get CNAME for stream %p", stream);
+      }
+      gst_structure_free (sdes);
+      g_object_unref (session);
+    } else {
+      GST_ERROR ("unable to get RTP session from stream %p", stream);
+    }
+  }
+
   gst_sdp_message_add_media (sdp, smedia);
   gst_sdp_media_free (smedia);
 
