@@ -84,6 +84,9 @@ struct _GstRTSPServerPrivate
   /* mount points for this server */
   GstRTSPMountPoints *mount_points;
 
+  /* request size limit */
+  guint content_length_limit;
+
   /* authentication manager */
   GstRTSPAuth *auth;
 
@@ -116,6 +119,7 @@ enum
 
   PROP_SESSION_POOL,
   PROP_MOUNT_POINTS,
+  PROP_CONTENT_LENGTH_LIMIT,
   PROP_LAST
 };
 
@@ -223,11 +227,23 @@ gst_rtsp_server_class_init (GstRTSPServerClass * klass)
           GST_TYPE_RTSP_MOUNT_POINTS,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * RTSPServer::content-length-limit:
+   *
+   * Define an appropriate request size limit and reject requests exceeding the
+   * limit.
+   *
+   * Since: 1.18
+   */
+  g_object_class_install_property (gobject_class, PROP_CONTENT_LENGTH_LIMIT,
+      g_param_spec_uint ("content-length-limit", "Limitation of Content-Length",
+          "Limitation of Content-Length",
+          0, G_MAXUINT, G_MAXUINT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_rtsp_server_signals[SIGNAL_CLIENT_CONNECTED] =
       g_signal_new ("client-connected", G_TYPE_FROM_CLASS (gobject_class),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTSPServerClass, client_connected),
-      NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      GST_TYPE_RTSP_CLIENT);
+      NULL, NULL, NULL, G_TYPE_NONE, 1, GST_TYPE_RTSP_CLIENT);
 
   klass->create_client = default_create_client;
 
@@ -248,6 +264,7 @@ gst_rtsp_server_init (GstRTSPServer * server)
   priv->backlog = DEFAULT_BACKLOG;
   priv->session_pool = gst_rtsp_session_pool_new ();
   priv->mount_points = gst_rtsp_mount_points_new ();
+  priv->content_length_limit = G_MAXUINT;
   priv->thread_pool = gst_rtsp_thread_pool_new ();
 }
 
@@ -417,7 +434,7 @@ gst_rtsp_server_set_service (GstRTSPServer * server, const gchar * service)
  *
  * Get the service on which the server will accept connections.
  *
- * Returns: (transfer full) (nullable): the service. use g_free() after usage.
+ * Returns: (transfer full): the service. use g_free() after usage.
  */
 gchar *
 gst_rtsp_server_get_service (GstRTSPServer * server)
@@ -600,6 +617,57 @@ gst_rtsp_server_get_mount_points (GstRTSPServer * server)
   return result;
 }
 
+/**
+ * gst_rtsp_server_set_content_length_limit
+ * @server: a #GstRTSPServer
+ * Configure @server to use the specified Content-Length limit.
+ *
+ * Define an appropriate request size limit and reject requests exceeding the
+ * limit.
+ *
+ * Since: 1.18
+ */
+void
+gst_rtsp_server_set_content_length_limit (GstRTSPServer * server, guint limit)
+{
+  GstRTSPServerPrivate *priv;
+
+  g_return_if_fail (GST_IS_RTSP_SERVER (server));
+
+  priv = server->priv;
+
+  GST_RTSP_SERVER_LOCK (server);
+  priv->content_length_limit = limit;
+  GST_RTSP_SERVER_UNLOCK (server);
+}
+
+/**
+ * gst_rtsp_server_get_content_length_limit:
+ * @server: a #GstRTSPServer
+ *
+ * Get the Content-Length limit of @server.
+ *
+ * Returns: the Content-Length limit.
+ *
+ * Since: 1.18
+ */
+guint
+gst_rtsp_server_get_content_length_limit (GstRTSPServer * server)
+{
+  GstRTSPServerPrivate *priv;
+  guint result;
+
+  g_return_val_if_fail (GST_IS_RTSP_SERVER (server), G_MAXUINT);
+
+  priv = server->priv;
+
+  GST_RTSP_SERVER_LOCK (server);
+  result = priv->content_length_limit;
+  GST_RTSP_SERVER_UNLOCK (server);
+
+  return result;
+}
+
 /**
  * gst_rtsp_server_set_auth:
  * @server: a #GstRTSPServer
@@ -739,6 +807,10 @@ gst_rtsp_server_get_property (GObject * object, guint propid,
     case PROP_MOUNT_POINTS:
       g_value_take_object (value, gst_rtsp_server_get_mount_points (server));
       break;
+    case PROP_CONTENT_LENGTH_LIMIT:
+      g_value_set_uint (value,
+          gst_rtsp_server_get_content_length_limit (server));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -766,6 +838,10 @@ gst_rtsp_server_set_property (GObject * object, guint propid,
     case PROP_MOUNT_POINTS:
       gst_rtsp_server_set_mount_points (server, g_value_get_object (value));
       break;
+    case PROP_CONTENT_LENGTH_LIMIT:
+      gst_rtsp_server_set_content_length_limit (server,
+          g_value_get_uint (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, propid, pspec);
   }
@@ -775,7 +851,7 @@ gst_rtsp_server_set_property (GObject * object, guint propid,
  * gst_rtsp_server_create_socket:
  * @server: a #GstRTSPServer
  * @cancellable: (allow-none): a #GCancellable
- * @error: (out): a #GError
+ * @error: a #GError
  *
  * Create a #GSocket for @server. The socket will listen on the
  * configured service.
@@ -1073,6 +1149,9 @@ default_create_client (GstRTSPServer * server)
   gst_rtsp_client_set_session_pool (client, priv->session_pool);
   /* set the mount points that this client should use */
   gst_rtsp_client_set_mount_points (client, priv->mount_points);
+  /* Set content-length limit */
+  gst_rtsp_client_set_content_length_limit (GST_RTSP_CLIENT (client),
+      priv->content_length_limit);
   /* set authentication manager */
   gst_rtsp_client_set_auth (client, priv->auth);
   /* set threadpool */
@@ -1138,6 +1217,7 @@ no_connection:
     GST_ERROR ("could not create connection from socket %p: %s", socket, str);
     g_free (str);
     g_object_unref (socket);
+    g_object_unref (client);
     return FALSE;
   }
 }
@@ -1238,8 +1318,8 @@ watch_destroyed (GstRTSPServer * server)
 /**
  * gst_rtsp_server_create_source:
  * @server: a #GstRTSPServer
- * @cancellable: (allow-none): a #GCancellable or %NULL.
- * @error: (out): a #GError
+ * @cancellable: (nullable): a #GCancellable or %NULL.
+ * @error: a #GError
  *
  * Create a #GSource for @server. The new source will have a default
  * #GSocketSourceFunc of gst_rtsp_server_io_func().
@@ -1300,7 +1380,7 @@ no_socket:
 /**
  * gst_rtsp_server_attach:
  * @server: a #GstRTSPServer
- * @context: (allow-none): a #GMainContext
+ * @context: (nullable): a #GMainContext
  *
  * Attaches @server to @context. When the mainloop for @context is run, the
  * server will be dispatched. When @context is %NULL, the default context will be
@@ -1347,7 +1427,7 @@ no_source:
 /**
  * gst_rtsp_server_client_filter:
  * @server: a #GstRTSPServer
- * @func: (scope call) (allow-none): a callback
+ * @func: (scope call) (nullable): a callback
  * @user_data: user data passed to @func
  *
  * Call @func for each client managed by @server. The result value of @func
