@@ -114,8 +114,8 @@ gst_rtsp_session_pool_class_init (GstRTSPSessionPoolClass * klass)
   gst_rtsp_session_pool_signals[SIGNAL_SESSION_REMOVED] =
       g_signal_new ("session-removed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstRTSPSessionPoolClass,
-          session_removed), NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE,
-      1, GST_TYPE_RTSP_SESSION);
+          session_removed), NULL, NULL, NULL, G_TYPE_NONE, 1,
+      GST_TYPE_RTSP_SESSION);
 
   klass->create_session_id = create_session_id;
   klass->create_session = create_session;
@@ -547,8 +547,7 @@ gst_rtsp_session_pool_cleanup (GstRTSPSessionPool * pool)
  * locked so no further actions on @pool can be performed from @func.
  *
  * If @func returns #GST_RTSP_FILTER_REMOVE, the session will be set to the
- * expired state with gst_rtsp_session_set_expired() and removed from
- * @pool.
+ * expired state and removed from @pool.
  *
  * If @func returns #GST_RTSP_FILTER_KEEP, the session will remain in @pool.
  *
